
#include <ap_int.h>
#include <ap_utils.h>
#include <hls_stream.h>


#if 0
typedef ap_int<16> i16_t;

typedef struct {
	ap_int<16> real;
	ap_int<16> imag;
} cint16;



typedef struct {
	ap_uint<32> data;
	ap_uint<1>  last;
} cplxi16stream;


#define SYNC 0x55AA

enum {
	STATE_SEARCH=0,
	STATE_SYNC,
	STATE_PASS
};

static ap_uint<2> state_machine = STATE_SEARCH;
static ap_uint<12> words_counter = 0;
static ap_int<16> real_save;
static bool to_write=false;


void input_interface(hls::stream<i16_t> &input, hls::stream<cplxi16stream> &output)
{
#pragma HLS INTERFACE ap_fifo port=input
#pragma HLS INTERFACE ap_fifo port=output
#pragma HLS RESOURCE variable=input  core=AXIS metadata="-bus_bundle S_AXIS"
#pragma HLS RESOURCE variable=output core=AXIS metadata="-bus_bundle M_AXIS"
#pragma HLS INTERFACE ap_ctrl_none port=return

	i16_t val = input.read();

#ifdef __SIMULATION
	fprintf(stderr, "state: %d", (int) state_machine);
#endif

	switch(state_machine){

	case STATE_SEARCH:
		if(val == SYNC) state_machine=STATE_SYNC;
		break;

	case STATE_SYNC:
		if(val == 2656 || val == 2552) {
			words_counter = val;
			state_machine=STATE_PASS;
		} else {
			 state_machine=STATE_SEARCH;
		}
		break;

	case STATE_PASS:
		if(!to_write) {
			to_write=true;
			real_save=val;
		} else {
			to_write=false;

			cint16 buff_in;
			buff_in.real=real_save;
			buff_in.imag=val;

			ap_uint<32> *dO = (ap_uint<32> *) &buff_in;
			cplxi16stream y;
			y.data = *dO;
			y.last=(words_counter == 0)?1:0;
			output.write(y);
			words_counter--;
			if(words_counter==0)
				state_machine=STATE_SEARCH;
		}

		break;

	}
#ifdef __SIMULATION
	fprintf(stderr, " => %d\n", (int) state_machine);
#endif

}

#else


#if 0

typedef struct {
	ap_uint<32> data;
	ap_uint<1>  last;
	ap_uint<1>  dest;
} cplxi16stream;


#define SYNC 0xB0BF
#define SYNC_STARTFRAME 0xACE0

enum {
	STATE_SEARCH=0,
	STATE_SYNC,
	STATE_PASS
};

static ap_uint<2> state_machine = STATE_SEARCH;
static ap_uint<12> words_counter = 0;
static ap_uint<16> word_save;


void input_interface(hls::stream<ap_uint<16> > &input, hls::stream<cplxi16stream> &output, ap_uint<2> *debug)
{
#pragma HLS INTERFACE ap_none port=debug
#pragma HLS INTERFACE ap_fifo port=input
#pragma HLS INTERFACE ap_fifo port=output
#pragma HLS RESOURCE variable=input  core=AXIS metadata="-bus_bundle S_AXIS"
#pragma HLS RESOURCE variable=output core=AXIS metadata="-bus_bundle M_AXIS"
#pragma HLS INTERFACE ap_ctrl_none port=return

	ap_uint<16> val = input.read();
	cplxi16stream y;

#ifdef __SIMULATION
	fprintf(stderr, "state: %d", (int) state_machine);
#endif

	switch(state_machine){

	case STATE_SEARCH:
		if(val == SYNC) {
			state_machine=STATE_SYNC;
		} else if(val == SYNC_STARTFRAME) {
			state_machine=STATE_SYNC;
			y.dest=1;
			y.last=1;
			y.data=0xFFFFFFFF;
			output.write(y);
			ap_wait();
			y.data=0;
			output.write(y);
			ap_wait();
		} else {
			*debug = 3;
		}
		break;

	case STATE_SYNC:
		if(val == 192) {
			words_counter = val;
			state_machine=STATE_PASS;
			*debug = 0;
		} else {
			state_machine=STATE_SEARCH;
			*debug = 1;
		}
		break;

	case STATE_PASS:
		//*debug = 0;

		if(words_counter.bit(0)==0) { //val%2==1
			word_save=val;
		} else {

			y.data = (((ap_uint<32>) val) << 16) | word_save;
			y.last=(words_counter == 1)?1:0;
			y.dest=0;
			output.write(y);
		}

		words_counter--;
		if(words_counter==0)
			state_machine=STATE_SEARCH;

		break;

	}
#ifdef __SIMULATION
	fprintf(stderr, " => %d\n", (int) state_machine);
#endif

}
#else


typedef struct {
	ap_uint<16> data;
	ap_uint<1>  last;
	ap_uint<1>  dest;
} cplxi16stream;


#define SYNC 0xB00B
#define SYNC_STARTFRAME 0xFACE

enum {
	STATE_SEARCH=0,
	STATE_SYNC,
	STATE_PASS
};

static ap_uint<2> state_machine = STATE_SEARCH;
static ap_uint<8> words_counter = 0;
static ap_uint<7> frames_counter = 0;


void input_interface(hls::stream<ap_uint<16> > &input, hls::stream<cplxi16stream> &output, ap_uint<2> *debug)
{
#pragma HLS INTERFACE ap_none port=debug
#pragma HLS INTERFACE ap_fifo port=input
#pragma HLS INTERFACE ap_fifo port=output
#pragma HLS RESOURCE variable=input  core=AXIS metadata="-bus_bundle S_AXIS"
#pragma HLS RESOURCE variable=output core=AXIS metadata="-bus_bundle M_AXIS"
#pragma HLS INTERFACE ap_ctrl_none port=return

	ap_uint<16> val = input.read();
	cplxi16stream y;

#ifdef __SIMULATION
	fprintf(stderr, "state: %d", (int) state_machine);
#endif

	switch(state_machine){

	case STATE_SEARCH:
		if(val == SYNC) {
			state_machine=STATE_SYNC;
			y.dest=1;
			y.last=1;
			y.data=0xFFFFFFFF;
			output.write(y);
		} else {
			*debug = 3;
		}
		break;

	case STATE_SYNC:
		if(val == SYNC_STARTFRAME) {
			y.dest=1;
			y.last=1;
			y.data=0;
			output.write(y);
			state_machine=STATE_PASS;
			words_counter=0;
			frames_counter=0;
			*debug = 0;
		} else {
			state_machine=STATE_SEARCH;
			*debug = 1;
		}
		break;

	case STATE_PASS:
		//*debug = 0;

		y.data = val;
		y.last=(words_counter == 1)?1:0;
		y.dest=0;
		output.write(y);

		words_counter++;
		if(words_counter==192) {
			words_counter=0;
			frames_counter++;
			if(frames_counter==75)
				state_machine=STATE_SEARCH;
		}

		break;

	}
#ifdef __SIMULATION
	fprintf(stderr, " => %d\n", (int) state_machine);
#endif

}

#endif

#endif
